#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2010-02-19
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: Fastqcheck.pm 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/lib/npg_qc/api/loader/Fastqcheck.pm $
#
package npg_qc::api::loader::Fastqcheck;
use strict;
use warnings;
use Moose;
use Carp qw{carp cluck croak confess};
use English qw{-no_match_vars};
use Readonly;
use Perl6::Slurp;
use Statistics::Lite qw(:all);

extends qw{npg_qc::api::loader::base};

Readonly::Scalar our $VERSION => do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mxs; $r; };

Readonly our $POSITION_OF_QVAL_TWENTY     => 26;
Readonly our $POSITION_OF_QVAL_TWENTYFIVE => 31;
Readonly our $POSITION_OF_QVAL_THIRTY     => 36;
Readonly our $POSITION_OF_QVAL            => 6;
Readonly our $PERCENTAGE_CALCULATOR       => 1000;

## no critic (Documentation::RequirePodAtEnd)

=head1 NAME

npg_qc::api::loader::Fastqcheck

=head1 VERSION

$Revision: 13723 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 file_list

fastqcheck file list

=cut
has 'file_list'    => (isa           => q{ArrayRef},
                       is            => q{rw},
                       lazy_build    => 1,
                       documentation => q{fastqcheck file name list},
                      );

sub _build_file_list {
  my $self = shift;

  my @file_list = ();

  my $folder_path = $self->archive_path();
  opendir my $dh, $folder_path or croak qq{Cannot open $folder_path};
  @file_list = grep { /[.]fastqcheck/mxs } sort readdir $dh;
  closedir $dh or croak qq(Cannot close $folder_path);

  return \@file_list;
}

=head2 run

=cut

sub run {
  my ($self) = @_;

  my $id_run = $self->id_run();

  $self->log( q{Loading Illumina fastqcheck files for Run } . $id_run . q{ into QC database} );

  my $file_list = $self->file_list();
  $self->log( q{There are } . scalar @{$file_list} . q{ files to assess} );

  my %files_per_lane;

  foreach my $fastqcheck_file ( @{ $file_list } ) {

    next if $fastqcheck_file =~ /human|phix/xms; # we don't want to archive the split statistics, just a full lane

    my $position;
    ( $position ) = $fastqcheck_file =~ m/${id_run}_(\d)/xms;

    $files_per_lane{ $position } ||= {
      files => [],
      spiked_phix_lane => 0,
    };

    push @{ $files_per_lane{ $position }->{files} }, $fastqcheck_file;

    ( $fastqcheck_file =~ /orig/xms ) && $files_per_lane{ $position }->{spiked_phix_lane}++;

  }

  my $count_of_fastqcheck_files = 0;

  foreach my $position ( sort keys %files_per_lane ) {

    foreach my $file ( @{ $files_per_lane{ $position }->{files} } ) {

      if (
          ! $files_per_lane{ $position }->{spiked_phix_lane}
            ||
          $file =~ /orig/xms
         ) {
            $self->_process_fastqcheck( {
              file => $file,
              position => $position
            } );
            $count_of_fastqcheck_files++;
      }

    }

  }

  $self->log( $count_of_fastqcheck_files . q{ files have had qcal scores loaded.} );

  $self->schema->txn_commit();

  return 1;
}

sub _process_fastqcheck {
    my ( $self, $arg_refs ) = @_;

    my $file_url = $arg_refs->{file};
    my $position = $arg_refs->{position};

    $self->log( $file_url );

    my $id_run = $self->id_run();
    my $end;

    my $orig_suffix = q{_orig};

    if ($file_url =~ /${id_run}_\d{1}_(\d{1}|t)(${orig_suffix}){0,1}[.]fastqcheck/xms) {
        ($end) = $file_url =~ /_\d{1}_(\d{1}|t)(${orig_suffix}){0,1}[.]fastqcheck/xms;
    } else {
        $end = 1;
    }

    my $fields_db = {
                      id_run   => $id_run,
                      position => $position,
                      end      => $end,
                    };

    $self->log("position: $position, end: $end");

    my $file = slurp $self->archive_path().qq{/$file_url};

    my @lines = split /\n/xms, $file;

    my @temp = split /\s+/xms, $lines[0];
    my $pf_bases = $temp[2];

    foreach my $line (@lines) {
      @temp = split /\s+/xms, $line;
      if ($temp[0] eq 'Total') {
        pop @temp;
        my @working_array = @temp;

        splice @working_array, 0, $POSITION_OF_QVAL;
        my $total = sum(@working_array);

        @working_array = @temp;
        splice @working_array, 0, $POSITION_OF_QVAL_TWENTY;
        my $perc_at_20plus = sum(@working_array) || 0;
        my $calculated_value = sprintf '%.00f', $pf_bases*$perc_at_20plus/$PERCENTAGE_CALCULATOR;
        $calculated_value ||= 0;
        $fields_db->{twenty} = $calculated_value;

        @working_array = @temp;
        splice @working_array, 0, $POSITION_OF_QVAL_TWENTYFIVE;
        my $perc_at_25plus = sum(@working_array) || 0;
        $calculated_value = sprintf '%.00f', $pf_bases*$perc_at_25plus/$PERCENTAGE_CALCULATOR;
        $calculated_value ||= 0;
        $fields_db->{twentyfive} = $calculated_value;

        @working_array = @temp;
        splice @working_array, 0, $POSITION_OF_QVAL_THIRTY;
        my $perc_at_30plus = sum(@working_array) || 0;
        $calculated_value = sprintf '%.00f', $pf_bases*$perc_at_30plus/$PERCENTAGE_CALCULATOR;
        $calculated_value ||= 0;
        $fields_db->{thirty} = $calculated_value;

        $self->schema->resultset('Qcal')->update_or_create( $fields_db );
        last;
      }
    }
    return 1;
  }
no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item MooseX::Getopt

=item npg_qc::api::loader::base

=item Perl6::Slurp

=item Statistics::Lite

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2009 Guoying Qi (gq1@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
