#########
# Author:        Marina Gourtovaia
# Maintainer:    $Author: ajb $
# Created:       29 July 2009
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: check.pm 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/lib/npg_qc/autoqc/checks/check.pm $
#

package npg_qc::autoqc::checks::check;

use strict;
use warnings;
use Moose;
use MooseX::ClassAttribute;
use Carp;
use English qw(-no_match_vars);
use File::Basename;
use File::Spec::Functions qw(catfile);
use File::Temp qw(tempdir);
use IPC::Open3;
use Perl6::Slurp;
use Symbol qw/gensym/;


use npg_qc::autoqc::types;

with qw/ npg_common::roles::run::lane::tag
         npg_common::roles::run::lane::file_names
       /;

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mxs; $r; };
## no critic (Documentation::RequirePodAtEnd ProhibitParensWithBuiltins ProhibitStringySplit)

=head1 NAME

npg_qc::autoqc::checks::check

=head1 VERSION

$Revision: 13723 $

=head1 SYNOPSIS

  my $check1 = npg_qc::autoqc::checks::check->new(path => q[a/valid/path], position => 1, id_run => 2222);

=head1 DESCRIPTION

A top-level class for autoqc checks. Checks are performed for one lane.

=head1 SUBROUTINES/METHODS

=cut


Readonly::Scalar our $FILE_EXTENSION  => 'fastq';


=head2 path

A path to a directory with fastq input files. Read-only.

=cut
has 'path'        => (isa      => 'Str',
                      is       => 'ro',
                      required => 1,
                     );

=head2 position

Lane number. An integer from 1 to 8 inclusive. Read-only.

=cut
has 'position'    => (isa       => 'LaneNumber',
                      is        => 'ro',
                      required  => 1,
                     );


=head2 id_run

Run id for the lane to be checked. Read-only.

=cut
has 'id_run'      => (
                       isa      => 'PositiveInt',
                       is       => 'ro',
                       required => 1,
                     );

=head2 sequence_type

Sequence type as phix for spiked phix or similar. Read-only.

=cut
has 'sequence_type'  => (
                         isa      => 'Maybe[Str]',
                         is       => 'ro',
                         required => 0,
		        );

=head2 tmp_path

A path to a directory that can be used by the check to write temporary files to.
By default, a temporary directory in a usual place for temporary files (/tmp on Linux)
will be created and deleted automatically on exit.

=cut
has 'tmp_path'    => (isa        => 'Str',
                      is         => 'ro',
                      required   => 0,
                      default    => sub { return tempdir(CLEANUP => 1); },
                     );

=head2 input_file_ext

Input file extension.

=cut
has 'input_file_ext' => (isa        => 'Str',
                         is         => 'ro',
                         required   => 0,
                         default    => $FILE_EXTENSION,
                         writer     => '_set_ext',
                        );


=head2 input_file

A ref to a list with names of input files for this check

=cut
has 'input_files'    => (isa        => 'ArrayRef',
                         is         => 'ro',
                         required   => 0,
                         lazy_build => 1,
                        );
sub _build_input_files {
    my $self = shift;
    my @files = $self->get_input_files();
    return \@files;
}


=head2 result

A result object. Read-only.

=cut
has 'result'     =>  (isa        => 'Object',
                      is         => 'ro',
                      required   => 0,
                      lazy_build => 1,
                     );
sub _build_result {

    my $self = shift;

    my $pkg_name = ref $self;
    my $module_version = $pkg_name->VERSION;

    my ($ref) = ($pkg_name) =~ /(\w*)$/smx;
    if ($ref eq q[check]) { $ref =  q[result]; }
    my $module = "npg_qc::autoqc::results::$ref";
    Class::MOP::load_class($module);

    my $result = $module->new(
                    id_run    => $self->id_run,
                    position  => $self->position,
                    path      => $self->path,
                    tag_index => $self->tag_index
                             );
    $result->set_info('Check', $pkg_name);
    $result->set_info('Check_version', $module_version);
    if ($result->can(q[sequence_type])) {
        $result->sequence_type($self->sequence_type);
    }
    return $result;
}


=head2 execute

The actual test should be performed within this method. In this class this method only checks that the given path exists.

=cut
sub execute {
    my $self = shift;
    if (!$self->path)     {croak q[No input files directory supplied];}
    if (!-e $self->path)  {croak q[Input files directory ] . $self->path() . q[ does not exist];}
    if (!@{$self->input_files}) {
        return 0;
    }
    return 1;
}


=head2 can_run

Decides whether this check can be run for a particular run.

=cut
sub can_run {
    return 1;
}


=head2 get_input_files

Returns an array containing full paths to a forward and reverse(if any) input files.

=cut
sub get_input_files {
    my $self = shift;

    my @fnames = ();
    #TODO: in future we might need to deal with run_lane_t.fastq for multiplexing
    my $forward = File::Spec->catfile($self->path, $self->generate_filename($self->input_file_ext, 1)->[0]);
    my $no_end_forward = undef;
    if (!-e $forward) {
        $no_end_forward = File::Spec->catfile($self->path, $self->generate_filename($self->input_file_ext)->[0]);
        if (-e $no_end_forward) {
           $forward = $no_end_forward;
        } else {
           my $bam_ext = q[bam];
           my $new_forward;
           if ($self->input_file_ext eq $bam_ext) {
                my $non_human_flag = 1;
                $new_forward = $self->nonsplit2split($no_end_forward, $non_human_flag, $bam_ext);
           }
           if (!$new_forward || !-e $new_forward) {
               $self->result->comments(qq[Neither $forward no $no_end_forward file found]);
               return @fnames;
           } else {
               $forward = $new_forward;
           }
        }
    }

    push @fnames, $forward;
    if (!defined $no_end_forward) {
        my $reverse =  File::Spec->catfile($self->path, $self->generate_filename($self->input_file_ext, 2)->[0]);
        if (-e $reverse) {push @fnames, $reverse;}
    }

    return @fnames;
}


=head2 generate_filename_attr

Gets an array containing paths to forward and reverse (if any) input files, and returns
an array ref with filenames that is suitable for setting the filename attribute.

=cut
sub generate_filename_attr {

    my ($self) = shift;

    my $count = 0;
    my $filename;
    foreach my $fname (@{$self->input_files}) {
        my($name, $directories, $suffix) = fileparse($fname);
        $filename->[$count] = $name;
        $count++;
    }
    return $filename;
}


=head2 write2file

Writes an argument text string to a folder given by the destination argument.
The output file name follows the pattern idrun_position.check_name.extension.
The default extension is xml.

=cut
sub write2file {

    my ($self, $string, $destination, $ext) = @_;

    my ($class_name) = (ref $self) =~ /(\w*)$/smx;
    ##no critic (RequireCheckedSyscalls)
    my $fname = sprintf q[%s_%s%s%s.%s.%s],
                   $self->id_run,
                   $self->position,
                   $self->tag_label(),
                   $self->type_label(),
                   $class_name,
                   $ext;

    $destination = catfile($destination, $fname);

    ##no critic (RequireBracedFileHandleWithPrint)
    my $fh = undef;
    open $fh, '>', $destination or croak "Cannot write to $destination";
    print $fh $string;
    close $fh or carp "Cannot close a handle to $destination";

    return 1;
}


=head2 aligner_version

Run aligner command to get version information

=cut
sub aligner_version {
    my ($self, $aligner_cmd) = @_;

    my $aligner_version;

    my $version_regex = qr{^
                            (?: $aligner_cmd )? \s*
                            version [:]? \s+
                            (\S+ (?: [ \t]+ \S+ )? )
                          }imsx;

    # Special pleading for stampy.
    if ( $aligner_cmd =~ m/ stampy $/msx ) {
        $aligner_cmd .= q{.py};
        $version_regex = qr{^ stampy \s+ (\S+) }imsx;

        # Special special pleading for Sanger.
        local $ENV{PATH} = '/software/python-2.7/bin:' . $ENV{PATH};
    }

    foreach my $arg ( q{}, '--version', '-v', '-V', 'version' ) {

        my $out;
        my $err;

        my $pid = open3( undef, $out, $err, "$aligner_cmd $arg" );

        my $output;
        if (defined $out) {
            $output = slurp($out);
            ($aligner_version) = $output =~ m/$version_regex/igmsx;
            close $out or croak q[Cannot close out stream];
        }

        if (defined $err) {
            if (!defined $aligner_version) {
                $output = slurp($err);
                ($aligner_version) = $output =~ m/$version_regex/igmsx;
	    }
            close $err or croak q[Cannot close error stream];
        }
        waitpid $pid, 0;

        last if defined $aligner_version;
    }

    ( defined $aligner_version ) ||
        ( $aligner_version = 'no version number obtained' );

    return $aligner_version;
}


=head2 overall_pass

Use this function to compute overall lane pass value for a particular check
if the evaluation is performed separately for a forward and a reverse sequence.

=cut
sub overall_pass {

  my ($self, $apass, $count) = @_;
  if ($apass->[0] != 1 || $count == 1) {return $apass->[0];}
  if ($apass->[1] != 1) {return $apass->[1];}
  return ($apass->[0] && $apass->[1]);
}


=head2 tag_label

Tag label as found in file names

=cut
sub tag_label {
    my $self = shift;
    return defined $self->tag_index ? q[#] . $self->tag_index : q[];
}


=head2 type_label

Type label as found in file names

=cut
sub type_label {
    my $self = shift;
    return $self->sequence_type ? q[_] . $self->sequence_type : q[];
}


no MooseX::ClassAttribute;
no Moose;

1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item MooseX::ClassAttribute

=item Carp

=item English -no_match_vars

=item File::Basename

=item File::Spec::Functions

=item File::Temp

=item IPC::Open3

=item npg_common::roles::run::lane::tag

=item npg_common::roles::run::lane::file_names

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Marina Gourtovaia E<lt>mg8@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Marina Gourtovaia

This file is part of NPG.

NPG is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut
