#########
# Author:        Marina Gourtovaia
# Maintainer:    $Author: ajb $
# Created:       9 August 2010
# Last Modified: $Date: 2011-07-21 16:13:49 +0100 (Thu, 21 Jul 2011) $
# Id:            $Id: file_store.pm 13723 2011-07-21 15:13:49Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/tags/release-38.0/lib/npg_qc/file_store.pm $
#

package npg_qc::file_store;

use strict;
use warnings;
use Moose;
use Carp;
use English qw{-no_match_vars};
use File::Spec;
use Perl6::Slurp;
use File::Basename;

use npg_qc::Schema;

with 'MooseX::Getopt';

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$Revision: 13723 $ =~ /(\d+)/mxs; $r; };

Readonly::Scalar our $DEFAULT_EXTENSION => q[.fastqcheck];
Readonly::Scalar our $TABLE_NAME        => q[Fastqcheck];

has 'path'            => (  isa         => 'ArrayRef',
                            is          => 'rw',
                            required    => 1,
                            documentation => 
  'a path to a directory with the files to be saved; multiple paths (an array) are accepted',
                         );

has 'extension'       => (  isa         => 'Str',
                            is          => 'rw',
                            required    => 0,
                            default     => $DEFAULT_EXTENSION,
                            documentation => 
  'a file extension (including a dot) of the files to be saved; defaults to .fastqcheck',
                         );

has '_schema' =>   ( isa        => 'npg_qc::Schema',
                     is         => 'ro',
                     required   => 0,
                     lazy_build => 1,
                   );
sub _build__schema {
    my $self = shift;
    my $schema = npg_qc::Schema->connect();
    return $schema;
}


sub _get_files {
  my ($self) = @_;

  my @all_files = ();
  foreach my $path (@{$self->path}) {   
    my @files = glob File::Spec->catfile($path, q[*] . $self->extension);
    if (@files) {
      push @all_files, @files;
    }
  }

  return \@all_files;
}

sub save_files {
  my $self = shift;

  my $count = 0;
  my $rs = $self->_schema->resultset($TABLE_NAME);
  foreach my $file (@{$self->_get_files}) {
    my $transaction = sub {
      my ($fname, $dir, $ext) = fileparse($file);
      my $text = slurp $file;
      $rs->update_or_create({file_name => $fname, file_content => $text,});
		      };
    eval {
      $self->_schema->txn_do($transaction);
      $count++;
      1;
    } or do {
      carp "Failed to load $file to the database: $EVAL_ERROR";
    };
  }

  return $count;
}

no Moose;
__PACKAGE__->meta->make_immutable;

1;
__END__

=head1 NAME

npg_qc::file_store

=head1 VERSION

$Revision: 13723 $

=head1 SYNOPSIS

 npg_qc::file_store->new(path => ['path1', 'path2'], extension => q[.json])->save_files();

=head1 DESCRIPTION

Saves files (default - .fastqcheck) found in the given paths into a database.

=head1 SUBROUTINES/METHODS

=head2 path - an attribute; a reference to an array of paths to directories with the files to be saved

=head2 extension - an attribute; a file extension (including a dot) of the files to be saved; defaults to .fastqcheck

=head2 save_files 

=head2 _schema - an attribute; DBIx schema object for the NPG QC database

head2 save_files - a method; finds files in the given paths, reads teh content and loads in to NPG QC database

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

Moose
MooseX::Getopt
Carp
English qw{-no_match_vars}
File::Spec
Perl6::Slurp
File::Basename
npg_qc::Schema

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Marina Gourtovaia, E<lt>mg8@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Marina Gourtovaia

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
